/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include <iostream>
#include <QFile>
#include <qtextstream.h>
#include "networkman.h"
#include "ui_networkman.h"
#include <kstandarddirs.h>
#include <pcbsd-netif.h>
#include <pcbsd-utils.h>
#include <qstring.h>

using namespace std;

void NetworkMan::Init()
{
    Changed = FALSE;
    
    colourWhite = QColor(255, 255, 255);
    colourRed = QColor(255, 78, 78);
    
    // Set tray icon disabled, since we may be running under root mode
    checkSysTray->setEnabled(FALSE);
    checkSysTray->setHidden(TRUE);
    
    
    // Let the list box know to add new devices
    firstUpdate = 0;
    
    // Get the username we're running under
    username = QString::fromLocal8Bit(getenv("LOGNAME"));
        
    connect(listNetDev, SIGNAL(itemSelectionChanged()), this, SLOT(DevSelectionChanged()) );
    connect(pushConfigure, SIGNAL(clicked()), this, SLOT(PropertiesSlot()) );
    connect(pushAdminConfig, SIGNAL(clicked()), this, SLOT(slotOpenAdminDlg()) );
    connect(checkSysTray, SIGNAL(clicked()), this, SLOT(slotSysTray()) );
    listNetDev->setContextMenuPolicy( Qt::CustomContextMenu );
    connect(listNetDev, SIGNAL(itemDoubleClicked( QListWidgetItem *) ), this, SLOT(slotDoubleClick(QListWidgetItem *) ) );
    connect(listNetDev, SIGNAL(customContextMenuRequested( const QPoint &) ), this, SLOT(slotListRightClick(const QPoint &) ) );
         
    
    // Load any global settings
    loadGlobals();
    
     // Start the device detection process
    detectDev();
    
}



void NetworkMan::detectDev()
{
    listNetDev->clear();
    textGlobalError->setText(tr("Detecting Devices...") );
    pushConfigure->setEnabled(FALSE);
    
    QStringList ifs = NetworkInterface::getInterfaces();
    uint i = 0;
    for ( QStringList::Iterator it = ifs.begin(); it != ifs.end(); ++it )
    {
       QString dev = *it;
       if (dev.indexOf("lo0") == -1
	   && dev.indexOf("fwe") == -1
	   && dev.indexOf("plip") == -1
	   && dev.indexOf("pfsync") == -1
	   && dev.indexOf("pflog") == -1
	   && dev.indexOf("tun") == -1)
       {
	   //qDebug("Found device: " + dev);
	   Devs[i] = dev;
	   // Determine if its a wireless or wired device
	   DevsType[i] = getTypeForIdent(Devs[i]);
	   // Get HW Idtentify line
	   DevsName[i] = getNameForIdent(Devs[i]);
	   
	   if ( DevsType[i] != "Wireless" ) comboPPPOEDev->addItem(Devs[i]);
      i++;
       }
   }
     textGlobalError->setText("");
     refreshDevices();
}


QString NetworkMan::getLineFromCommandOutput( QString command )
{
	FILE *file = popen(command.toLatin1(),"r"); 
	
	char buffer[100];
	
	QString line = ""; 
	char firstChar;
	
	if ((firstChar = fgetc(file)) != -1){
		line += firstChar;
		line += fgets(buffer,100,file);
	}
	
	
	pclose(file);
	
	
	return line;
}


QString NetworkMan::getNameForIdent( QString ident )
{
    NetworkInterface ifr(ident);
    return ifr.desc();
}


QString NetworkMan::getIpForIdent( QString ident )
{
   NetworkInterface ifr(ident);
   return ifr.ipAsString();
}


QString NetworkMan::getMacForIdent( QString ident )
{
   NetworkInterface ifr(ident);
   return ifr.macAsString();
}


QString NetworkMan::getStatusForIdent( QString ident )
{
   NetworkInterface ifr(ident);
   return ifr.mediaStatusAsString();
}

void NetworkMan::DevSelectionChanged()
{
    int    sel = listNetDev->currentRow();
    
    if ( sel != -1 ) {
    
    pushConfigure->setEnabled(TRUE);

	
    // Check to see if the network tray icon is enabled or not
     QString filename = "/home/" + username + "/.kde4/Autostart/tray-" + Devs[sel] + ".desktop";


  // First run a check to if we need to enable or disable the checkbox
  if ( QFile::exists( filename ) )
  {
     checkSysTray->setChecked(TRUE);
   } else {
     checkSysTray->setChecked(FALSE);
   }
    
    
   DevsIP[sel] = getIpForIdent(Devs[sel]);
   DevsStatus[sel] = getStatusForIdent(Devs[sel]);
   DevsNetmask[sel] = getNetmaskForIdent(Devs[sel]);
    
   textStatusLabel1->setText("Address:");	
   textStatus1->setText(tr("IP: ") + DevsIP[sel] + " / " + tr("Netmask: ") + DevsNetmask[sel]);
    
   textStatusLabel2->setText("MAC Address:");
   textStatus2->setText(DevsMAC[sel]);
   
   textStatusLabel3->setText("Status:");
   textStatus3->setText(DevsStatus[sel]);
    
  }
    
}


bool NetworkMan::checkValue( QString File, QString Key, QString Value )
{
QFile file( File );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            
               // If the KEY is found in the line, continue processing 
	if ( line.indexOf(Key, 0) != -1)
	{
	    line.remove(line.indexOf(Key, 0), Key.length());
	    if ( line.indexOf(Value, 0) != -1)
	    {
		return TRUE;
	    }
	    
	}
	    
	    
	    
        }
        file.close();
    }
    
    return FALSE;
}




void NetworkMan::PropertiesSlot()
{
    QString program;
    QStringList arguments;
    
    workingDev = listNetDev->currentRow();
    
    if ( workingDev != -1) {
    
    if ( DevsType[workingDev] == "Wireless" )
    {
	// If we are running as root
	if ( getuid() != 0 )
	{
          program = "kdesu";
	  arguments.clear();
	  arguments << "-d" << "--noignorebutton" << "pc-wificonfig" << Devs[workingDev];
	 } else {
          program = "pc-wificonfig";
	  arguments.clear();
	  arguments << Devs[workingDev];
	 }

	 ConfigDevice = new QProcess( this );
         ConfigDevice->start(program, arguments); 

    } else {
   
	// If we are running as root
	if ( getuid() != 0 )
	{
          program = "kdesu";
	  arguments.clear();
	  arguments << "-d" << "--noignorebutton" << "pc-ethernetconfig" << Devs[workingDev];
	 } else {
          program = "pc-ethernetconfig";
	  arguments.clear();
	  arguments << Devs[workingDev];
	 }

	 ConfigDevice = new QProcess( this );
         ConfigDevice->start(program, arguments); 

   }
    
  }
     
}





void NetworkMan::runCommand( QString command )
{
	FILE *file = popen(command.toLatin1(),"r"); 
	fclose(file);
}


QString NetworkMan::getValue( QString File, QString Key, int occur )
{
     int found = 1;
    
    QFile file( File );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            
               // If the KEY is found in the line, continue processing 
	if ( line.indexOf(Key, 0) != -1 && line.indexOf("#", 0) != 0)
	{
	    if ( found == occur) {
	    
	    line.remove(line.indexOf(Key, 0), Key.length());
	       
	    
	    // Remove any quotes
	    if ( line.indexOf('"') == 0 )
	    {
		line = line.remove(0, 1);
	    }
	    
	    if ( line.indexOf('"') != -1  )
	    {
		line.truncate(line.indexOf('"'));
	    }

	    return line;
	    
	    } else {
	       found++;  
	    }
	    
	    
	}
	    
	    
	    
        }
        file.close();
    }
    
    return "";
    
}


void NetworkMan::saveValue( QString File, QString oldKey, QString newKey, int occur )
{
    // Lets the dev save a value into a specified config file. The occur value tells which occurance of "oldKey" to replace
    // If occur is set to -1, it will remove any duplicates of "oldKey"
    
    
    QString SavedFile;

    int found = 1;
    int wroteKey = 0;
    
    int firstLine = 0;
    
    if ( occur == -1 && newKey.isEmpty() )
    {
	
    }
    
    
    // Load the old file, find the oldKey, remove it and replace with newKey
    QFile file( File );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            
            // If the KEY is found in the line and this matches the occurance that must be processed
            if ( line.indexOf(oldKey, 0) != -1 && wroteKey == 0 && found == occur && line.indexOf("#", 0) != 0)
           {
	   if ( firstLine == 0) {
		
	    SavedFile = SavedFile + newKey;
	    found++;
	    firstLine = 1;
	    } else {
	    SavedFile = SavedFile + "\n" + newKey;
	    found++;	
	    wroteKey = 1;
	    }
	    
	  // If the KEY is found, and we are just on wrong occurance, save it and continue to search  
            } else if (line.indexOf(oldKey, 0) != -1 && occur != -1 && found != occur && line.indexOf("#", 0) != 0 ) {
	    if ( firstLine == 0) {
	         SavedFile = SavedFile + line ;
                        firstLine = 1;
	     } else {
	         SavedFile = SavedFile + "\n" + line ;     
                   }
	     
	found++;
	
	
             // if we are trying to remove the key, don't save it	
            } else if (line.indexOf(oldKey, 0) != -1 && occur == -1 && newKey.isEmpty() && line.indexOf("#", 0) != 0 ) {
	
             // We just want one occurance of the key
            } else if (line.indexOf(oldKey, 0) != -1 && occur == -1 && ! newKey.isEmpty() && line.indexOf("#", 0) != 0 ) {
	   if ( firstLine == 0) {
	       SavedFile = SavedFile + newKey;
	       found++;
	       firstLine = 1;
	    } else {
		SavedFile = SavedFile + "\n" + newKey;
		found++;	
		wroteKey = 1;
	    }
	    
	    
	// String not found at all
            } else {
	    
	    if ( firstLine == 0) {
	         SavedFile = SavedFile + line ;
                        firstLine = 1;
	     } else {
	         SavedFile = SavedFile + "\n" + line ;     
                   }
	}
	    
	    
	    
        }
	// Didnt find the key? Write it!
	if ( wroteKey == 0 && ! newKey.isEmpty())
	{
	    SavedFile = SavedFile + "\n" + newKey;
	}
	
        file.close();
    }
    
    
    // Save the new file
    QFile fileout( File );
    if ( fileout.open( QIODevice::WriteOnly ) ) {
        QTextStream streamout( &fileout );
                    streamout << SavedFile;
        fileout.close();
    }
    
}


void NetworkMan::loadGlobals()
{    
    QString tmp;
    lineHostname->setText(getValue("/etc/rc.conf", "hostname=", 1));
    lineGateway->setText(getValue("/etc/rc.conf", "defaultrouter=", 1) );
    lineDNS1->setText(getValue("/etc/resolv.conf", "nameserver", 1) );
    lineDNS2->setText(getValue("/etc/resolv.conf", "nameserver", 2) );

    // Check for IPv6 settings
    tmp = getValue("/etc/rc.conf", "ipv6_enable=", 1) ;
    lineIPv6Gateway->setText(getValue("/etc/rc.conf", "ipv6_defaultrouter=", 1) );
    
    
    
    // Load the PPPOE Configuration

    if ( QFile::exists( "/etc/ppp/ppp.conf" ) )
   {
       loadPPPOEConfig();
   }
	
}





QString NetworkMan::getNetmaskForIdent( QString ident )
{
   NetworkInterface ifr(ident);
   return ifr.netmaskAsString();
}




bool NetworkMan::checkRange( QString IP )
{
    
    int num;
    
    num = IP.section(".", 0,0).toInt();
    
    if ( num > 255 || num < 0)
    {
	return false;
    }
    
   num = IP.section(".", 1,1).toInt();
    
    if ( num > 255 || num < 0)
    {
	return false;
    }
    
    
    num = IP.section(".", 2,2).toInt();
    
    if ( num > 255 || num < 0)
    {
	return false;
    }
    
   num = IP.section(".", 3,3).toInt();
    
    if ( num > 255 || num < 0)
    {
	return false;
    }
    
    return TRUE;
    

}


void NetworkMan::restartNetwork()
{
    
   //restartDialog= new restartPopup(this);
   //restartDialog->init();
   //restartDialog->exec();
   
    //Set the hostname
    runCommand("hostname " + lineHostname->text());

   // Set the gateway device name
   if ( lineGateway->text() != "..."  ) {
	runCommand("route delete default" );
	runCommand("route add default " + lineGateway->text() );
   }

}


QString NetworkMan::getTypeForIdent( QString ident )
{
   NetworkInterface ifr(ident);
   if (ifr.isWireless()) return "Wireless";
   return "Ethernet";
}


void NetworkMan::refreshDevices()
{
    
    int i = 0;
    listNetDev->clear(); 
    while (! Devs[i].isEmpty() )
    {
	
                DevsIP[i] = getIpForIdent(Devs[i]);
		DevsStatus[i] = getStatusForIdent(Devs[i]);
		DevsUp[i] = getUpStatusForIdent(Devs[i]);
		DevsNetmask[i] = getNetmaskForIdent(Devs[i]);
		
		if ( firstUpdate == 0) {
		  DevsMAC[i] = getMacForIdent(Devs[i]);
		  listNetDev->setCurrentRow(0);
	        }

		if ( DevsType[i] == "Wireless")
		{
		    
		    QPixmap PixmapIcon(KStandardDirs::locate("data", "pc-netmanager/pics/network_wifi.png"));
		    listNetDev->addItem(new QListWidgetItem(PixmapIcon.scaled(32,32), Devs[i] + " (" +DevsName[i] + ")") );
		    
		 } else {
		    // If we are looking at a regular Ethernet device  
		    QPixmap PixmapIcon(KStandardDirs::locate("data", "pc-netmanager/pics/network_local.png"));
		    listNetDev->addItem(new QListWidgetItem(PixmapIcon.scaled(32,32), Devs[i] + " (" +DevsName[i] + ")") );
		}
	
	i++;
    }
			   
}


void NetworkMan::slotSysTray()
{
    int    sel = listNetDev->currentRow();
    if ( sel == -1 )
    {
       return;	
    }
    
    QString Type;
    QString Command;
    
    // Start the tray applet
    if (checkSysTray->isChecked() )
    { 
	
	if ( DevsType[sel] == "Wireless" )
	{
	   Type="-wireless";  
	} else {
	    Type="-ethernet";
	}
	
	    // Save the new file
                 QFile fileout( "/home/" + username + "/.kde4/Autostart/tray-" + Devs[sel] + ".desktop" );
                 if ( fileout.open( QIODevice::WriteOnly ) ) {
                          QTextStream streamout( &fileout );
                                streamout << "[Desktop Entry]\nExec=pc-nettray ";
		    streamout << Type + " " + Devs[sel] ;
		    streamout << "\nIcon=network\nStartupNotify=false\nType=Application\n";
                                fileout.close();
		    
		    runCommand("chown " + username + ":" + username + " \"/home/" + username + "/.kde4/Autostart/tray-" + Devs[sel] + ".desktop\"");
		    //runCommand("su " + username + " -c \"pc-nettray " + Type + " " + Devs[sel] + "\"");
		    // Run the command without SU flags
		   runCommand("pc-nettray " + Type + " " + Devs[sel] + " &");
		    
                    }
	
    } else {
	// Kill the tray applet
	runCommand("rm -f \"/home/" + username + "/.kde4/Autostart/tray-" + Devs[sel] + ".desktop\"");
    }

}


void NetworkMan::slotDoubleClick( QListWidgetItem *item )
{
    // Start the configuration dialog
    PropertiesSlot();
}


void NetworkMan::loadPPPOEConfig()
{
    
    
     QFile filein( "/etc/ppp/ppp.conf" );
     int foundProfile = 0;
     QString ProfileName = "internet";
     QString tmp, tmp2;
     
     // Check to see if PPPoE is enabled
     tmp = getValue( "/etc/rc.conf", "ppp_enable=", 1 );
     if ( tmp != "YES" ) {
       return; 
     }




   	// Start loading the rc.conf information
   	tmp = getValue( "/etc/rc.conf", "ppp_mode=", 1 );
	if ( tmp == "ddial" ) {
		checkPPPAlwaysOn->setChecked(TRUE);
	}
   	tmp = getValue( "/etc/rc.conf", "ppp_nat=", 1 );
	if ( tmp == "YES" ) {
		checkPPPNat->setChecked(TRUE);
	}

	// Now load the ppp.conf file
	if ( filein.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &filein );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'

	    // Check if we are on a profile line
	    if ( line.indexOf(":") == line.length() -1  ) {
		if ( line.indexOf("default") == -1 ) {
		    // If we find the existing profile for this device, remove it
		    foundProfile = 1;
		    
		} else {
		    foundProfile = 0;
		}
	    }

	    if ( foundProfile == 1)
	    {
		// Search for the username and load it
		if ( line.indexOf("authname") != -1 )
		{
			tmp = line.remove(0, line.indexOf("authname") + 9 );
			linePPPUsername->setText(tmp);
		}
		// Search for the password and load it
		if ( line.indexOf("authkey") != -1 )
		{
			tmp = line.remove(0, line.indexOf("authkey") + 8 );
			linePPPPassword->setText(tmp);
		}
		
		// Search for the device / service tag
		if ( line.indexOf("set device PPPoE:") != -1 )
		{
		    tmp = line.remove(0, line.indexOf("set device PPPoE:") + 18 );
		    
		    // Search for the service tag in remainder line
		    if ( tmp.indexOf(":") != -1 ) {
			tmp2 = tmp.remove(0, tmp.indexOf(":") + 1 );
			linePPPService->setText(tmp2);
			tmp.truncate( tmp.indexOf(":") );
		    }
		    
		    // Now look for device string
		   /* int i = 0;
		    while (! Devs[i].isEmpty() ) {
			if ( tmp == Devs[i] ) {
			    comboPPPOEDev->setCurrentItem(i);
			}
			i++;
		    }
		   */
		}
	    }

	} 
    } 

}


void NetworkMan::slotTimerRefresh()
{
    QTimer::singleShot( 10000, this, SLOT(refreshDevices()) );
}




void NetworkMan::slotListRightClick( const QPoint &pos )
{
    int currentItem = listNetDev->currentRow();
    if ( currentItem != -1 ) {
	  popup = new KMenu();
	  popup->addAction( tr("Configure ") + Devs[currentItem], this, SLOT(PropertiesSlot()));
	  popup->addAction( tr("Device Information"), this, SLOT(slotShowInfoConfig()));
	  // If we are running as root
	  if ( getuid() == 0 )
	  {
	  popup->addSeparator();
	  if ( DevsUp[currentItem] == "DOWN" ) {
	    popup->addAction( tr("Enable device"), this, SLOT(slotEnableDevice()));
                } else {
	    popup->addAction( tr("Disable device"), this, SLOT(slotDisableDevice()));
                }
                popup->addSeparator();
                popup->addAction( tr("Restart the Network"), this, SLOT(restartNetwork()));
	  }
	  popup->exec( QCursor::pos() );
    }

}


void NetworkMan::slotDisableDevice()
{
        int currentItem = listNetDev->currentRow();
    if ( currentItem != -1 ) {
	runCommand("ifconfig " + Devs[currentItem] + " down");
    }

}




void NetworkMan::slotEnableDevice()
{
        int currentItem = listNetDev->currentRow();
    if ( currentItem != -1 ) {
	runCommand("ifconfig " + Devs[currentItem] + " up");		
    }	
}


void NetworkMan::slotShowInfoConfig()
{
   QString program;
   QStringList arguments;

   workingDev = listNetDev->currentRow();
    
   if ( workingDev != -1) {
    
    if ( DevsType[workingDev] == "Wireless" )
    {

	// If we are running as root
	if ( getuid() != 0 )
	{
          program = "kdesu";
	  arguments.clear();
	  arguments << "-d" << "--noignorebutton" << "pc-wificonfig" << "info" << Devs[workingDev];
	 } else {
          program = "pc-wificonfig";
	  arguments.clear();
	  arguments << "info" << Devs[workingDev];
	 }

	 ConfigDevice = new QProcess( this );
         ConfigDevice->start(program, arguments); 

    } else {
   
	// If we are running as root
	if ( getuid() != 0 )
	{
          program = "kdesu";
	  arguments.clear();
	  arguments << "-d" << "--noignorebutton" << "pc-ethernetconfig" << "info" << Devs[workingDev];
	 } else {
          program = "pc-ethernetconfig";
	  arguments.clear();
	  arguments << "info" << Devs[workingDev];
	 }

	 ConfigDevice = new QProcess( this );
         ConfigDevice->start(program, arguments); 
   }
    
  }
}


void NetworkMan::setNotRoot()
{
    // Disable functionality since we are not running as root
    checkSysTray->setEnabled(TRUE);
    checkSysTray->setHidden(FALSE);
    pushConfigure->setEnabled(FALSE);
    lineDNS1->setEnabled(FALSE);
    lineDNS2->setEnabled(FALSE);
    lineHostname->setEnabled(FALSE);
    lineGateway->setEnabled(FALSE);
    lineIPv6Gateway->setEnabled(FALSE);
    
    linePPPUsername->setEnabled(FALSE);
    linePPPPassword->setEnabled(FALSE);
    linePPPService->setEnabled(FALSE);
    comboPPPOEDev->setEnabled(FALSE);
    checkPPPAlwaysOn->setEnabled(FALSE);
    checkPPPNat->setEnabled(FALSE);
    
}


QString NetworkMan::getUpStatusForIdent( QString ident )
{
   NetworkInterface ifr(ident);
   if (ifr.isUp()) return "UP";
   return "DOWN";
}

void NetworkMan::slotOpenAdminDlg()
{
     QString program;
     QStringList arguments;

     // If we are running as root
     if ( getuid() != 0 )
     {
          program = "kdesu";
	  arguments.clear();
          arguments << "-d" << "--noignorebutton" << "pc-netadmin";
     } else {
          program = "pc-netadmin";
	  arguments.clear();
     }

     ConfigSys = new QProcess( this );
     connect( ConfigSys, SIGNAL(processExited()), this, SLOT(loadGlobals() ) );
     ConfigSys->start(program, arguments); 
       
}
