
/* Qt */
#include <qtimer.h>
#include <QProcess>
#include <QFile>
#include <qtooltip.h>
#include <qimage.h>
#include <qtranslator.h>
#include <qtextstream.h>
#include <qmenu.h>

/* KDE */
#include <kiconloader.h>
#include <kpassivepopup.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <iostream>

#include "NetworkTray.h"

/* Update interval in ms */
#define  UPDATE_MSEC 4000

// Public Variables
QString DeviceName;
QString DeviceType;
QString DeviceIP;
QString DeviceIPv6;
QString DeviceMAC;
QString DeviceSSID;
QString DeviceSignalStrength = "";
int 	DeviceSavedStrength = 0;
QString DeviceWirelessSpeed = "";
QString DeviceStatus;
QString DeviceUpStatus;
QString DeviceIdent;
QString DeviceNetmask;
QString DeviceMedia;
QString DeviceGateway;
QString username;


void NetworkTray::programInit(QString Type, QString Device)
{
    // Connect our double-click slot
    connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(slotTrayActivated(QSystemTrayIcon::ActivationReason)));

    QString tmp;
    QIcon Icon;
    DeviceName = Device;
    DeviceType = Type;

    username = QString::fromLocal8Bit(getenv("LOGNAME"));
    

    DeviceStatus = getStatusForIdent( DeviceName );
    DeviceIP = getIpForIdent( DeviceName );
    DeviceIPv6 = getIPv6ForIdent( DeviceName);
    DeviceMAC = getMacForIdent( DeviceName );
    DeviceStatus = getStatusForIdent( DeviceName );
    DeviceIdent = getNameForIdent( DeviceName );
    DeviceNetmask = getNetmaskForIdent( DeviceName );
    DeviceUpStatus = getUpStatusForIdent( DeviceName );
    DeviceMedia = getMediaForIdent( DeviceName );
    DeviceGateway = getGatewayForIdent( DeviceName );

//QMessageBox::information( this, "Application name",  "IP: " );


    if ( Type == "Ethernet" )
    {
        if ( (DeviceStatus == "active" || DeviceStatus == "") && DeviceUpStatus == "UP" )
        {
	   Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_local.png"));
	} else {
    	   Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_local_dis.png"));
        }
    } else {
        if ( DeviceStatus == "associated" && DeviceUpStatus == "UP" )
        {
	   Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi.png"));
	} else {
    	   Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi_dis.png"));
        }
        DeviceSSID = getSSIDForIdent(DeviceName);
        if (! DeviceSSID.isEmpty() )
	{
	  DeviceSignalStrength = getSignalStrengthForIdent( DeviceName );
	  DeviceWirelessSpeed = getWirelessSpeedForIdent(DeviceName);
	}
    }


  // Add configuration button to the context menu
  setIcon(Icon);
  contextMenu()->clear();
  contextMenu()->addSeparator();
  contextMenu()->addAction( tr("Configure ") + DeviceName, this, SLOT(openConfigDlg()));
  contextMenu()->addAction( tr("Device Information "), this, SLOT(openDeviceInfo()));
  contextMenu()->addAction( tr("Start the Network Manager"), this, SLOT(openNetManager()));
  contextMenu()->addSeparator();
  contextMenu()->addAction( tr("Restart the Network"), this, SLOT(slotRestartNetwork()));

  // Check if we are on a ethernet device and PPPOE is enabled
     if ( Type == "Ethernet" )
    {
        // Check for PPPOE configuration
	tmp = getValue( "/etc/rc.conf", "ppp_enable=", 1 );
	if ( tmp == "YES") {
 	  QFile filein( "/etc/ppp/ppp.conf" );
       	 
     	  QString ProfileName = "internet";
          
	  // Now load the ppp.conf file
	  if ( filein.open( QIODevice::ReadOnly ) ) {
          QTextStream stream( &filein );
          QString line;
          while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'

	    // Check if we are on a profile line
	    if ( line.indexOf(":") == line.length() -1  ) {
		if ( line.indexOf(ProfileName) != -1 ) {
		    // If we find the existing profile for this device, add the context menu shortcuts
		     contextMenu()->addSeparator();
  		     contextMenu()->addAction( tr("Connect to PPPoE service."), this, SLOT(slotEnablePPPOE()));
		     contextMenu()->addAction( tr("Disconnect from PPPoE service."), this, SLOT(slotDisablePPPOE()));

		} 
	    }


	}
     }

     } // end of TMP = YES line

   } // End of PPPOE check
  
  // Create the tooltip popup now
  displayTooltip();

  // Start the monitor service
  QTimer::singleShot(UPDATE_MSEC,  this,  SLOT(monitorStatus()));

}

QString NetworkTray::getLineFromCommandOutput( QString command )
{
	FILE *file = popen(command.toLatin1(),"r"); 
	
	char buffer[100];
	
	QString line = ""; 
	char firstChar;
	
	if ((firstChar = fgetc(file)) != -1){
		line += firstChar;
		line += fgets(buffer,100,file);
	}
	
	
	pclose(file);
	
	
	return line;
}


QString NetworkTray::getNameForIdent( QString ident )
{
	
	QString command = "dmesg | grep " + ident;
	QString inputLine = getLineFromCommandOutput(command);
	QString name = "";
	
	inputLine = inputLine.remove (0, inputLine.indexOf("<") + 1);
	inputLine.truncate(inputLine.indexOf(">"));
	name = inputLine;
	
	return name;	
}


QString NetworkTray::getIpForIdent( QString ident )
{
	QString command = "nice ifconfig " +ident + " | grep \"inet \"";
	QString inputLine = getLineFromCommandOutput(command);
	QString ip= "";
	
	if (inputLine != "" && inputLine.indexOf("inet ") != -1){
		
		ip = inputLine.remove(0, inputLine.indexOf("inet ") + 5);
		ip.truncate(ip.indexOf(" "));
		
		
	}
	
	return ip;
}


QString NetworkTray::getMacForIdent( QString ident )
{
    
    	QString command = "ifconfig " +ident + " | grep \"ether\"";
	QString inputLine = getLineFromCommandOutput(command);
	QString ether = "";
	
	if (inputLine != "" && inputLine.indexOf("ether") != -1){
		
		ether = inputLine.remove(0, inputLine.indexOf(" ") + 1);
	
	}
	
	return ether;

}

QString NetworkTray::getSSIDForIdent( QString ident )
{
    	QString command = "nice ifconfig " +ident + " | grep \"ssid\"";
	QString inputLine = getLineFromCommandOutput(command);
	QString SSID = "";
	
	if (inputLine != "" && inputLine.indexOf("ssid ") != -1){
	 
		SSID = inputLine.remove(0, inputLine.indexOf(" ") + 1);
		SSID.truncate(inputLine.indexOf("channel") - 1 );
	}
	
	return SSID;
	


}

QString NetworkTray::getStatusForIdent( QString ident )
{
    	QString command = "nice ifconfig " +ident + " | grep \"status\"";
	QString inputLine = getLineFromCommandOutput(command);
	QString status = "";
	
	if (inputLine != "" && inputLine.indexOf("status") != -1){
		
		status = inputLine.remove(0, inputLine.indexOf(" ") + 1);
		status.truncate(inputLine.indexOf("\n") );
	
	}
	
	return status;
	


}

QString NetworkTray::getNetmaskForIdent( QString ident )
{
        
    int num;
    QString RealIP, tmp;
    
	QString command = "nice ifconfig " +ident + " | grep \"netmask\"";
	QString inputLine = getLineFromCommandOutput(command);
	QString net= "";
	
	if (inputLine != "" && inputLine.indexOf("netmask ") != -1){
		
		net = inputLine.remove(0, inputLine.indexOf("netmask ") + 8);
		net.truncate(net.indexOf(" "));
		
		num = net.mid(2, 2).toInt(0, 16);
		tmp.setNum(num);
		RealIP = tmp;
		
		num = net.mid(4, 2).toInt(0, 16);
		tmp.setNum(num);
		RealIP = RealIP + "." + tmp ;
		
		num = net.mid(6, 2).toInt(0, 16);
		tmp.setNum(num);
		RealIP = RealIP + "." + tmp ;
		
		num = net.mid(8, 2).toInt(0, 16);
		tmp.setNum(num);
		RealIP = RealIP + "." + tmp ;
		
		//QMessageBox::information( this, "Application name",  "IP: " + RealIP );
		
	}
	
	return RealIP;
    
}

QString NetworkTray::getSignalStrengthForIdent( QString ident )
{
	
	QString command = "ifconfig " + ident + " scan | grep " + DeviceSSID;
	QString line = getLineFromCommandOutput(command);
        QString tmp;
	
        // Get the signal strength of this device
        tmp = line;
        tmp = tmp.simplified();

        // First remove the SSID which may have spaces in it
        tmp.remove(0, tmp.indexOf(":"));

        // Now find the strength section
        tmp = tmp.section(" ", 3, 3);
        tmp.truncate(tmp.indexOf(":"));

	return tmp;	
}

QString NetworkTray::getWirelessSpeedForIdent( QString ident )
{
	
	QString command = "ifconfig " + ident + " scan | grep " + DeviceSSID;
	QString line = getLineFromCommandOutput(command);
        QString tmp;

        // Get the signal strength of this device
        tmp = line;
        tmp = tmp.simplified();

        // First remove the SSID which may have spaces in it
        tmp.remove(0, tmp.indexOf(":"));

        // Now find the Speed section
        tmp = tmp.section(" ", 2, 2);
        return tmp;
}

void NetworkTray::slotTrayActivated(QSystemTrayIcon::ActivationReason reason) {
   if(reason == QSystemTrayIcon::DoubleClick) {
      openNetManager();
   }
}


void  NetworkTray::openConfigDlg() {

	if ( DeviceType == "Ethernet" )
	{
           QString program = "kdesu";
           QStringList arguments;
           arguments << "--noignorebutton" << "-d" << "pc-ethernetconfig " + DeviceName;
	   QProcess *runCommandProc = new QProcess(this);
           runCommandProc->start(program, arguments);
	} else {
	   QString program = "kdesu";
           QStringList arguments;
           arguments << "--noignorebutton" << "-d" << "pc-wificonfig " + DeviceName;
	   QProcess *runCommandProc = new QProcess(this);
           runCommandProc->start(program, arguments);
	}

}

void  NetworkTray::displayTooltip() {


   QString  tooltipStr;

   tooltipStr = QString(tr("Device Name:") + " /dev/" + DeviceName);

   tooltipStr += "<br>" + DeviceIdent +"<hr>";


       if ( DeviceType == "Ethernet" )
       {
            if ( DeviceStatus == "active" || DeviceStatus == "" ) {
   		tooltipStr += "<br>" + tr("IP:") + " " + DeviceIP;
	        tooltipStr += "<br>" + tr("IPv6:") + " " + DeviceIPv6;
		tooltipStr += "<br>" + tr("Netmask:") + " " + DeviceNetmask;
		tooltipStr += "<br>" + tr("Gateway:") + " " + DeviceGateway;
		tooltipStr += "<br>" + tr("Mac/Ether:") + " " + DeviceMAC;
		tooltipStr += "<br>" + tr("Media:") + " " + DeviceMedia;
   		tooltipStr += "<br>" + tr("Status:") + " " + DeviceUpStatus;
	    } else {
   		tooltipStr += "<br>" + tr("Mac/Ether:") + " " + DeviceMAC;
		tooltipStr += "<br>" + tr("Media:") + " " + DeviceMedia;
   		tooltipStr += "<br>" + tr("Status:") + " " + DeviceStatus;
		tooltipStr += "<br>" + tr("No connection detected.<br> Check your cable connection and try again!");
	    }

       } else {
	// If this is a wireless device, give different output
            if ( DeviceStatus == "associated" ) {
		tooltipStr += "<br>" + tr("IP:") + " " + DeviceIP;
		tooltipStr += "<br>" + tr("IPv6:") + " " + DeviceIPv6;
		tooltipStr += "<br>" + tr("Netmask:") + " " + DeviceNetmask;
		tooltipStr += "<br>" + tr("Gateway:") + " " + DeviceGateway;
      		tooltipStr += "<br>" + tr("SSID:") + " " + DeviceSSID;
      		tooltipStr += "<br>" + tr("Connection Strength:") + " " + DeviceSignalStrength + "%";
      		tooltipStr += "<br>" + tr("Connection Speed:") + " " + DeviceWirelessSpeed;
		tooltipStr += "<br>" + tr("Mac/Ether:") + " " + DeviceMAC;
		tooltipStr += "<br>" + tr("Media:") + " " + DeviceMedia;
   		tooltipStr += "<br>" + tr("Status:") + " " + DeviceStatus;
	    } else {
		tooltipStr += "<br>" + tr("Mac/Ether:") + " " + DeviceMAC;
		tooltipStr += "<br>" + tr("Media:") + " " + DeviceMedia;
   		tooltipStr += "<br>" + tr("Status:") + " " + DeviceStatus;
		tooltipStr += "<br>" + tr("No wireless connection detected.<br> Double-click to start the wireless configuration wizard.");
	    }
       }

   // Update the tooltop
   setToolTip(tooltipStr);

   
}

void  NetworkTray::monitorStatus() {
  // Start checking to see if the device has changed, and if it has inform the user
  QString tmp;  
  int updateTooltip = 0;
  int displayPassivePopup = 0;
  QIcon Icon;
  QIcon Icon2;
  QString NotifyText = "";
  QString filename = "/home/" + username + "/.kde4/Autostart/tray-" + DeviceName + ".desktop";

  // First run a check to see if this applet has been disabled in the network manager
  QFile testFile(filename);
  if ( ! testFile.exists() )
  {
    exit(0);
  }

  // Check the media status of this device
  tmp = getMediaForIdent( DeviceName );
  if ( DeviceMedia != tmp )
  {
     // Save the status as the new status now
     DeviceMedia = tmp;
     updateTooltip = 1;
  }

  // Check for Gateway Changes
  tmp = getGatewayForIdent( DeviceName);
  if ( DeviceGateway != tmp )
  {
     NotifyText+="Gateway changed to: " + tmp + "<br>";
     displayPassivePopup = 1;
     // Save the status as the new status now
     DeviceGateway= tmp;
     updateTooltip = 1;
  }


  // Check for IPv6 Changes
  tmp = getIPv6ForIdent( DeviceName);
  if ( DeviceIPv6 != tmp )
  {
     NotifyText+="IPv6 changed to: " + tmp + "<br>";
     displayPassivePopup = 1;
     // Save the status as the new status now
     DeviceIPv6 = tmp;
     updateTooltip = 1;
  }

  // Get the device up status
  tmp = getUpStatusForIdent( DeviceName );
  if ( DeviceUpStatus != tmp )
  {
     NotifyText+="Link status changed to: " + tmp + "<br>";

     if ( DeviceType == "Ethernet" )
     {
         if ( (DeviceStatus == "active" || DeviceStatus == "")  && tmp == "UP") {
            Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_local.png"));
         } else {
            Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_local_dis.png"));
	 }

     } else {
         if ( DeviceStatus == "associated" && tmp == "UP" ) {
          Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi.png"));
	 } else {
          Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi_dis.png"));

         }
     }

     
     setIcon(Icon);


     // Save the status as the new status now
     DeviceUpStatus = tmp;
     updateTooltip = 1;
     displayPassivePopup = 1;
  }


  // First check the status of the device
  tmp = getStatusForIdent(DeviceName);
  if ( DeviceStatus != tmp )
  {
       

       if ( DeviceType == "Ethernet" )
       {
            if ( (tmp == "active" || tmp == "")  && DeviceUpStatus == "UP") {
             Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_local.png"));
	    } else {
             Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_local_dis.png"));
	    }

       } else {
            if ( tmp == "associated" && DeviceUpStatus == "UP" ) {
             Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi.png"));
	    } else {
             Icon.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi_dis.png"));
	     DeviceSavedStrength = 0;
	    }
       }

     // Set the tray icon now
     setIcon(Icon);

     NotifyText+="Link status changed to: " + tmp + "<br>";

     // Save the status as the new status now
     DeviceStatus = tmp;
     updateTooltip = 1;
     displayPassivePopup = 1;
  }

  // Now check the IP Address for changes
  tmp = getIpForIdent(DeviceName);
  if ( DeviceIP != tmp )
  {
     NotifyText+="IP address changed to: " + tmp + "<br>";
     // Save the IP as the new IP now
     DeviceIP = tmp;
     updateTooltip = 1;
     displayPassivePopup = 1;
  }

  tmp = getNetmaskForIdent( DeviceName );
  if ( DeviceNetmask != tmp )
  {
     NotifyText+="Netmask changed to: " + tmp + "<br>";
     // Save the IP as the new IP now
     DeviceNetmask = tmp;
     updateTooltip = 1;
     displayPassivePopup = 0;
  }


   if ( DeviceType == "Wireless" && DeviceStatus == "associated" )
   {
        // Now check the SSID Address for changes
        tmp = getSSIDForIdent(DeviceName);
        if ( DeviceSSID != tmp )
        {
             NotifyText+="SSID changed to: " + tmp + "<br>";
             // Save the SSID as the new SSID now
             DeviceSSID = tmp;
             updateTooltip = 1;
             displayPassivePopup = 1;
        }

	if (! DeviceSSID.isEmpty() )
	{
	  int newStrength = 0;
          int iconStrength = 0;
          QString FileLoad;
	  bool ok;
	  // Get the strength of the signal
	  tmp = getSignalStrengthForIdent( DeviceName );

	  // Figure out if we need to change the strength icon
	  tmp.toInt(&ok);
          if ( ok ) {
	  newStrength = tmp.toInt(&ok);
	  if ( newStrength < 5 )
          {	
		iconStrength = 0;
		FileLoad=KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi.png");
          } else if ( newStrength < 50 ) {
		iconStrength = 30;
		FileLoad=KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi30.png");
	  }  else if ( newStrength < 75 ) {
		iconStrength = 60;
		FileLoad=KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi60.png");
	  } else {
		iconStrength = 85;
		FileLoad=KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi85.png");
	  }

	  // If the strength has changed enough, change the loaded icon
  	  if ( DeviceSavedStrength != iconStrength  )
	  {
		QIcon Icon3;
	 	DeviceSavedStrength = iconStrength;

             	Icon3.addFile(FileLoad);
     	     	setIcon(Icon3);
	  }

          }
          DeviceSignalStrength = tmp; 
          updateTooltip = 1;

          // Get the connection speed being used
          DeviceWirelessSpeed = getWirelessSpeedForIdent( DeviceName );
          updateTooltip = 1;
	}

   }


   if ( updateTooltip == 1)
   {
	// Show the notification that something important changed
      if ( displayPassivePopup == 1) {

      Icon2.addFile(KStandardDirs::locate("data", "pc-netmanager/pics/info.png"));
 
     	KPassivePopup::message("Device " + DeviceName + ":", NotifyText,  Icon2.pixmap(22,22),  this);
      }

      displayTooltip();
   }


     // First run a check to see if this applet has been disabled in the network manager
    if ( ! QFile::exists( filename ) )
    {
      close(0);
    }

   QTimer::singleShot(UPDATE_MSEC,  this,  SLOT(monitorStatus()));
}

// If the user wants to restart the network, do so
void NetworkTray::slotRestartNetwork() {
    QString program = "kdesu";
    QStringList arguments;
    arguments << "--noignorebutton" << "-d" << "/etc/rc.d/netif restart ; /etc/rc.d/routing restart ; /etc/rc.d/ppp restart";
    QProcess *runCommandProc = new QProcess(this);
    runCommandProc->start(program, arguments);
}

void NetworkTray::slotEnablePPPOE() {
    QString ProfileName = "internet";
    QString program = "kdesu";
    QStringList arguments;
    arguments << "--noignorebutton" << "-d" << "ppp -background " + ProfileName;
    QProcess *runCommandProc = new QProcess(this);
    runCommandProc->start(program, arguments);
}
void NetworkTray::slotDisablePPPOE() {
    QString program = "kdesu";
    QStringList arguments;
    arguments << "--noignorebutton" << "-d" << "kilalll ppp";
    QProcess *runCommandProc = new QProcess(this);
    runCommandProc->start(program, arguments);
}

void NetworkTray::openNetManager() {
    QString program = "kcmshell4";
    QStringList arguments;
    arguments << "pcnetwork";
    QProcess *runCommandProc = new QProcess(this);
    runCommandProc->start(program, arguments);
}

QString NetworkTray::getValue( QString File, QString Key, int occur ) {

int found = 1;
    
    QFile file( File );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            
               // If the KEY is found in the line, continue processing 
	if ( line.indexOf(Key, 0) != -1 && line.indexOf("#", 0) != 0)
	{
	    if ( found == occur) {
	    
	    line.remove(line.indexOf(Key, 0), Key.length());
	       
	    
	    // Remove any quotes
	    if ( line.indexOf('"') == 0 )
	    {
		line = line.remove(0, 1);
	    }
	    
	    if ( line.indexOf('"') != -1  )
	    {
		line.truncate(line.indexOf('"'));
	    }

	    return line;
	    
	    } else {
	       found++;  
	    }
	    
	    
	}
	    
	    
	    
        }
        file.close();
    }
    
    return "";
}

void  NetworkTray::openDeviceInfo() {

        QString program = "kdesu";
        QStringList arguments;
	if ( DeviceType == "Ethernet" )
	{
           arguments << "--noignorebutton" << "-d" << "pc-ethernetconfig info " + DeviceName;
	} else {
           arguments << "--noignorebutton" << "-d" << "pc-wificonfig info " + DeviceName;
	}
        QProcess *runCommandProc = new QProcess(this);
        runCommandProc->start(program, arguments);

}

QString NetworkTray::getUpStatusForIdent( QString ident )
{
        	QString command = "nice ifconfig " +ident + " | grep \"flags=\"";
	QString inputLine = getLineFromCommandOutput(command);
	QString status = "";
	
	if (inputLine != "" && inputLine.indexOf("<UP") != -1){
	    status = "UP";
	} else {
	    status = "DOWN";  
	}
	
	return status;
    
}

QString NetworkTray::getMediaForIdent( QString ident )
{
    	QString command = "nice ifconfig " +ident + " | grep \"media:\"";
	QString inputLine = getLineFromCommandOutput(command);
	QString status = "";
	
	if (inputLine != "" && inputLine.indexOf("media:") != -1){
		
		status = inputLine.remove(0, inputLine.indexOf("media:") + 7);
		status.truncate(inputLine.indexOf("\n") );
	
	}
	
	return status;
	


}

QString NetworkTray::getIPv6ForIdent( QString ident )
{
	QString command = "nice ifconfig " +ident + " | grep \"inet6 \"";
	QString inputLine = getLineFromCommandOutput(command);
	QString ip= "";
	
	if (inputLine != "" && inputLine.indexOf("inet6 ") != -1){
		
		ip = inputLine.remove(0, inputLine.indexOf("inet6 ") + 5);
		ip.truncate(ip.indexOf("%"));
		
		
	}
	
	return ip;
}

QString NetworkTray::getGatewayForIdent( QString ident )
{
        QString command = "nice netstat -n -r | grep \"default\" | grep " + ident;
	QString inputLine = getLineFromCommandOutput(command);
	QString packets = "";
	
	if (inputLine != "" ){
		inputLine = inputLine.simplified();
		packets = inputLine.section(' ', 1, 1 );
	}
	
	return packets;
}

