/***************************************************************************
 *   Copyright (C) 2007 by Kris Moore   *
 *   kris@pcbsd.com   *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. *
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/


#include <qlabel.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qgroupbox.h>
#include <qfile.h>
#include <qtabwidget.h>
#include <iostream>
#include <qtranslator.h>
#include <qdesktopwidget.h>
#include <qapplication.h>
#include <qtimer.h>
#include <qpushbutton.h>
#include <qtextstream.h>

#include <iostream>

#include <pcbsd-netif.h>
#include <pcbsd-utils.h>

#include "ethernetconfig.h"
#include "ui_ethernetconfig.h"


using namespace std;

QString DeviceName;



void ethernetconfig::slot_close()
{

   exit(0);

}

bool ethernetconfig::checkRange(QString IP)
{
 int num;
    
    num = IP.section(".", 0,0).toInt();
    
    if ( num > 255 || num < 0)
    {
	return false;
    }
    
   num = IP.section(".", 1,1).toInt();
    
    if ( num > 255 || num < 0)
    {
	return false;
    }
    
    
    num = IP.section(".", 2,2).toInt();
    
    if ( num > 255 || num < 0)
    {
	return false;
    }
    
   num = IP.section(".", 3,3).toInt();
    
    if ( num > 255 || num < 0)
    {
	return false;
    }
    
    return TRUE;

}

void ethernetconfig::slot_apply()
{

    if ( ! buttonApply->isEnabled() ) {
	return;
    }


	if ( checkDisableNetwork->isChecked() )
	{
		saveValue( "/etc/rc.conf", "ifconfig_" + DeviceName + "=", "", -1);
		runCommand("ifconfig " + DeviceName + " down");
                buttonApply->setEnabled(FALSE);
		return;
	}


    if (! checkDHCP->isChecked() )
    {
	if ( lineIP->text() == "..." || lineNetmask->text() == "..." )
	{
	    QMessageBox::information( this, tr("Missing Fields"),
                    tr("You must enter an IP and Netmask to continue!\n") );
	    
	    return;
	}
	
	if ( ! checkRange(lineIP->text() ) ) {
	    QMessageBox::warning( this, tr("Warning"),
	    tr("IP Address is out of range! (") + lineIP->text() + tr(") Fields must be between 0-255.") );
	   return;
               }
	
	if ( ! checkRange(lineNetmask->text() ) ) {
	    QMessageBox::warning( this, tr("Warning"),
	    tr("Netmask is out of range! (") + lineNetmask->text() + tr(") Fields must be between 0-255.") );
	   return;
               }

        if ( checkMAC->isChecked() ) {
          saveValue( "/etc/rc.conf", "ifconfig_" + DeviceName, "ifconfig_" + DeviceName + "=\"inet " + lineIP->text() + " netmask " + lineNetmask->text() + "\"", -1);
        } else {
          saveValue( "/etc/rc.conf", "ifconfig_" + DeviceName, "ifconfig_" + DeviceName + "=\"inet " + lineIP->text() + " netmask " + lineNetmask->text() + " ether " + lineMAC->text() +"\"", -1);
        }
	  

    } else {
         if ( checkMAC->isChecked() ) {
	// Set the device to DHCP mode
        saveValue( "/etc/rc.conf", "ifconfig_" + DeviceName + "=", "ifconfig_" + DeviceName + "=\"DHCP\"", -1);
        } else {
	saveValue( "/etc/rc.conf", "ifconfig_" + DeviceName + "=", "ifconfig_" + DeviceName + "=\"DHCP ether " + lineMAC->text() + "\"", -1);
        }
    }


    if ( checkIPv6Enable->isChecked() ) {
       // Get rid of the ifconfig line for the ipv6 interface
       saveValue( "/etc/rc.conf", "ipv6_ifconfig_" + DeviceName + "=", "", -1);
    } else {
       saveValue( "/etc/rc.conf", "ipv6_ifconfig_" + DeviceName + "=", "ipv6_ifconfig_" + DeviceName + "=\"" + lineIPv6Address->text() + "\"", -1);
    }



    if ( ! checkMAC->isChecked() )
    {
       runCommand("ifconfig " + DeviceName + " ether " + lineMAC->text() );
    }

    //runCommand("/etc/rc.d/netif restart &");
    //runCommand("/etc/rc.d/routing restart &");

     if ( checkDHCP->isChecked() )
    {
         runCommand("dhclient " + DeviceName + " &");
    } else {
         runCommand("ifconfig " + DeviceName + " inet " + lineIP->text() );
	 runCommand("ifconfig " + DeviceName + " netmask " + lineNetmask->text() );
    }

    if ( ! checkIPv6Enable->isChecked() )
    {
	runCommand("ifconfig " + DeviceName + " inet6 " + lineIPv6Address->text() );
    }

   // Done, now set the apply button to off
   buttonApply->setEnabled(FALSE);

}

void ethernetconfig::slotIPCheckbox()
{
   if ( checkDHCP->isChecked() ) {
       groupBoxIP->setEnabled(FALSE);
   } else {
	groupBoxIP->setEnabled(TRUE);
   }

   // Done, now set the apply button to on
   buttonApply->setEnabled(TRUE);
}

void ethernetconfig::programInit()
{
   // See if we need to enable / disable the IP setup
   slotIPCheckbox();
   // Done, now set the apply button to on
   buttonApply->setEnabled(FALSE);

   // Connect our buttons / slots
   connect(buttonClose, SIGNAL(clicked()), this, SLOT(slot_close()) );
   connect(buttonApply, SIGNAL(clicked()), this, SLOT(slot_apply()) );
   connect(checkDisableNetwork, SIGNAL(clicked()), this, SLOT(slotDisableCheck()) );
   connect(checkDHCP, SIGNAL(clicked()), this, SLOT(slotIPCheckbox()) );
   connect(buttonOK, SIGNAL(clicked()), this, SLOT(slotOK()) );
   connect(checkMAC, SIGNAL(clicked()), this, SLOT(slotMacClicked()) );
   connect(lineIP, SIGNAL(textChanged(const QString &)), this, SLOT(slotCheckGlobalText(const QString &) ) );
   connect(lineNetmask, SIGNAL(textChanged(const QString &)), this, SLOT(slotCheckGlobalText(const QString &) ) );
   connect(lineIPv6Address, SIGNAL(textChanged(const QString &)), this, SLOT(slotCheckGlobalText(const QString &) ) );
   connect(lineMAC, SIGNAL(textChanged(const QString &)), this, SLOT(slotCheckGlobalText(const QString &) ) );

}

void ethernetconfig::saveValue( QString File, QString oldKey, QString newKey, int occur )
{
    // Lets the dev save a value into a specified config file. The occur value tells which occurance of "oldKey" to replace
    // If occur is set to -1, it will remove any duplicates of "oldKey"
    
    
    QString SavedFile;

    int found = 1;
    int wroteKey = 0;
    
    int firstLine = 0;
    
    if ( occur == -1 && newKey.isEmpty() )
    {
	
    }
    
    
    // Load the old file, find the oldKey, remove it and replace with newKey
    QFile file( File );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            
            // If the KEY is found in the line and this matches the occurance that must be processed
            if ( line.indexOf(oldKey, 0) != -1 && wroteKey == 0 && found == occur && line.indexOf("#", 0) != 0)
           {
	   if ( firstLine == 0) {
		
	    SavedFile = SavedFile + newKey;
	    found++;
	    firstLine = 1;
	    } else {
	    SavedFile = SavedFile + "\n" + newKey;
	    found++;	
	    wroteKey = 1;
	    }
	    
	  // If the KEY is found, and we are just on wrong occurance, save it and continue to search  
            } else if (line.indexOf(oldKey, 0) != -1 && occur != -1 && found != occur && line.indexOf("#", 0) != 0 ) {
	    if ( firstLine == 0) {
	         SavedFile = SavedFile + line ;
                        firstLine = 1;
	     } else {
	         SavedFile = SavedFile + "\n" + line ;     
                   }
	     
	found++;
	
	
             // if we are trying to remove the key, don't save it	
            } else if (line.indexOf(oldKey, 0) != -1 && occur == -1 && newKey.isEmpty() && line.indexOf("#", 0) != 0 ) {
	
             // We just want one occurance of the key
            } else if (line.indexOf(oldKey, 0) != -1 && occur == -1 && ! newKey.isEmpty() && line.indexOf("#", 0) != 0 ) {
	   if ( firstLine == 0) {
	       SavedFile = SavedFile + newKey;
	       found++;
	       firstLine = 1;
	    } else {
		SavedFile = SavedFile + "\n" + newKey;
		found++;	
		wroteKey = 1;
	    }
	    
	    
	// String not found at all
            } else {
	    
	    if ( firstLine == 0) {
	         SavedFile = SavedFile + line ;
                        firstLine = 1;
	     } else {
	         SavedFile = SavedFile + "\n" + line ;     
                   }
	}
	    
	    
	    
        }
	// Didnt find the key? Write it!
	if ( wroteKey == 0 && ! newKey.isEmpty())
	{
	    SavedFile = SavedFile + "\n" + newKey;
	}
	
        file.close();
    }
    
    
    // Save the new file
    QFile fileout( File );
    if ( fileout.open( QIODevice::WriteOnly ) ) {
        QTextStream streamout( &fileout );
                    streamout << SavedFile;
        fileout.close();
    }

}

void ethernetconfig::setDevice(QString Device)
{
   
   buttonApply->setEnabled(FALSE);

   DeviceName = Device;


   this->setWindowTitle(DeviceName + " " + tr("configuration") );

   // Connect the slot to load the values
   QTimer::singleShot(100,  this,  SLOT(slotFinishLoad()));

   
  

}

void ethernetconfig::runCommand( QString command )
{
	FILE *file = popen(command.toLatin1(),"r"); 
	fclose(file);
}

QString ethernetconfig::getValue( QString File, QString Key, int occur )
{

int found = 1;
    
    QFile file( File );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            
               // If the KEY is found in the line, continue processing 
	if ( line.indexOf(Key, 0) != -1 && line.indexOf("#", 0) != 0)
	{
	    if ( found == occur) {
	    
	    line.remove(line.indexOf(Key, 0), Key.length());
	       
	    
	    // Remove any quotes
	    if ( line.indexOf('"') == 0 )
	    {
		line = line.remove(0, 1);
	    }
	    
	    if ( line.indexOf('"') != -1  )
	    {
		line.truncate(line.indexOf('"'));
	    }

	    return line;
	    
	    } else {
	       found++;  
	    }
	    
	    
	}
	    
	    
	    
        }
        file.close();
    }
    
    return "";
}


void ethernetconfig::slotDisableCheck()
{
  if ( checkDisableNetwork->isChecked() )
  {
	tabMainWidget->setEnabled(FALSE);
  } else {
	tabMainWidget->setEnabled(TRUE);
  }
   // Done, now set the apply button to on
   buttonApply->setEnabled(TRUE);
}

void ethernetconfig::slotOK()
{
  slot_apply();
  exit(0);
}

void ethernetconfig::slotIPv6clicked()
{
	if ( checkIPv6Enable->isChecked() ) {
		groupIPv6->setEnabled(FALSE);
	} else {
		groupIPv6->setEnabled(TRUE);
	}

   // Done, now set the apply button to on
   buttonApply->setEnabled(TRUE);
}

void ethernetconfig::slotMacClicked()
{
   if ( checkMAC->isChecked() ) {
	groupMAC->setEnabled(FALSE);
   } else {
	groupMAC->setEnabled(TRUE);
   }
   // Done, now set the apply button to on
   buttonApply->setEnabled(TRUE);
}

void ethernetconfig::loadInfo()
{
  // Get all the device information for the info tab
  if ( textInfoName->text().isEmpty() ) {
  textInfoName->setText(getNameForIdent(DeviceName) );
  }


  textIP->setText(getIpForIdent(DeviceName) );
  textNetmask->setText(getNetmaskForIdent( DeviceName ) );
  textMac->setText(getMacForIdent( DeviceName ) );
  textStatus->setText(getStatusForIdent( DeviceName ) );
  textIPv6->setText(getIPv6ForIdent( DeviceName) );
  textGateway->setText(getGatewayForIdent(DeviceName) );
  textMedia->setText(getMediaForIdent(DeviceName) );

  // Get the packet status for this device
  textPacketsIn->setText(getInPacketsForIdent( DeviceName) );
  textPacketsInErrors->setText( getInErrorsForIdent( DeviceName) );
  textPacketsOut->setText(getOutPacketsForIdent( DeviceName) );
  textPacketsOutErrors->setText(getOutErrorsForIdent( DeviceName) );

  // Connect the slot to refresh
  QTimer::singleShot(3000,  this,  SLOT(loadInfo()));
}

QString ethernetconfig::getNameForIdent( QString ident )
{
    NetworkInterface ifr(ident);
    return ifr.desc();
}

QString ethernetconfig::getIpForIdent( QString ident )
{
	NetworkInterface ifr(ident);
   return ifr.ipAsString();
}

QString ethernetconfig::getMacForIdent( QString ident )
{
   NetworkInterface ifr(ident);
   return ifr.macAsString();
}

QString ethernetconfig::getStatusForIdent( QString ident )
{
   NetworkInterface ifr(ident);
   return ifr.mediaStatusAsString();
}

QString ethernetconfig::getInPacketsForIdent(QString ident)
{
   NetworkInterface ifr(ident);
   return QString::number(ifr.packetsRx());
}

QString ethernetconfig::getInErrorsForIdent(QString ident)
{
   NetworkInterface ifr(ident);
   return QString::number(ifr.errorsRx());
}

QString ethernetconfig::getOutPacketsForIdent(QString ident)
{
   NetworkInterface ifr(ident);
   return QString::number(ifr.packetsTx());
}

QString ethernetconfig::getOutErrorsForIdent(QString ident)
{
   NetworkInterface ifr(ident);
   return QString::number(ifr.errorsTx());
}

QString ethernetconfig::getNetmaskForIdent( QString ident )
{
   NetworkInterface ifr(ident);
   return ifr.netmaskAsString();
}

QString ethernetconfig::getLineFromCommandOutput( QString command )
{
	FILE *file = popen(command.toLatin1(),"r"); 
	
	char buffer[100];
	
	QString line = ""; 
	char firstChar;
	
	if ((firstChar = fgetc(file)) != -1){
		line += firstChar;
		line += fgets(buffer,100,file);
	}
	
	
	pclose(file);
	
	
	return line;
}

void ethernetconfig::showInfo()
{
	tabMainWidget->setCurrentWidget( tabMainWidget->widget(2) );
}

void ethernetconfig::slotFinishLoad()
{


   QString tmp, tmp2;
   // Start loading the device information
   tmp = getValue( "/etc/rc.conf", "ifconfig_" + DeviceName + "=", 1 );


   if ( tmp.isEmpty() ) 
   {
	checkDisableNetwork->setChecked(TRUE);
	tabMainWidget->setEnabled(FALSE);
   } else if ( tmp.indexOf("DHCP") != -1 )
   {
      checkDHCP->setChecked(TRUE);
      lineNetmask->setText("255.255.255.0");
      slotIPCheckbox();

      // Look for the mac address change
	if ( tmp.indexOf("ether") != -1 ) {
		tmp2 = tmp;
		tmp2 = tmp2.remove(0, tmp.indexOf("ether") + 6 );
		//if ( tmp.indexOf(" ") != -1 ) {
		//  tmp2.truncate(tmp.indexOf(" ") );
		//}
		lineMAC->setText(tmp2);
		checkMAC->setChecked(FALSE);
		groupMAC->setEnabled(TRUE);
	} 

   } else {
      	if ( tmp.indexOf("inet") == 0)
	{
	        // Get the IP address first
		tmp2 = tmp;
		tmp2 = tmp2.remove(0, tmp.indexOf("inet") + 5 );
		//tmp2 = tmp2.remove(0, tmp.indexOf(" ") );
		tmp2.truncate(tmp.indexOf("netmask") -1 );
		lineIP->setText(tmp2);

		// Look for the netmask
		if ( tmp.indexOf("netmask") != -1 ) {
			tmp2 = tmp;
			tmp2.simplified();
			tmp2 = tmp2.remove(0, tmp.indexOf("netmask") + 8 );
  			if ( tmp.indexOf("ether") != -1 ) {
			tmp2.truncate(tmp.indexOf("ether") );
			}
			lineNetmask->setText(tmp2);
		} else {
		   lineNetmask->setText("255.255.255.0");
		}

		// Look for the mac address change
		if ( tmp.indexOf("ether") != -1 ) {
			tmp2 = tmp;
			tmp2 = tmp2.remove(0, tmp.indexOf("ether") + 6 );
			//if ( tmp.indexOf(" ") != -1 ) {
			//  tmp2.truncate(tmp.indexOf(" ") );
			//}
			lineMAC->setText(tmp2);
			checkMAC->setChecked(FALSE);
			groupMAC->setEnabled(TRUE);
		} 

	}


   }


  // Check for the IPv6 Configuration
  // Start loading the device information
   tmp = getValue( "/etc/rc.conf", "ipv6_ifconfig_" + DeviceName + "=", 1 );
   if ( tmp.isEmpty() ) {
        checkIPv6Enable->setChecked(TRUE);
   } else {
	checkIPv6Enable->setChecked(FALSE);
	lineIPv6Address->setText(tmp);
   }

  slotIPv6clicked();

  // Starts grabbing information for the info tab
  loadInfo();


  buttonApply->setEnabled(FALSE);
}

QString ethernetconfig::getIPv6ForIdent(QString ident)
{
	QString command = "nice ifconfig " +ident + " | grep \"inet6 \"";
	QString inputLine = getLineFromCommandOutput(command);
	QString ip= "";
	
	if (inputLine != "" && inputLine.indexOf("inet6 ") != -1){
		
		ip = inputLine.remove(0, inputLine.indexOf("inet6 ") + 5);
		ip.truncate(ip.indexOf("%"));
		
		
	}
	
	return ip;
}

QString ethernetconfig::getGatewayForIdent(QString ident)
{
        QString command = "nice netstat -n -r | grep \"default\" | grep " + ident;
	QString inputLine = getLineFromCommandOutput(command);
	QString packets = "";
	
	if (inputLine != "" ){
		inputLine = inputLine.simplified();
		packets = inputLine.section(' ', 1, 1 );
	}
	
	return packets;
}

QString ethernetconfig::getMediaForIdent( QString ident )
{
    	QString command = "nice ifconfig " +ident + " | grep \"media:\"";
	QString inputLine = getLineFromCommandOutput(command);
	QString status = "";
	
	if (inputLine != "" && inputLine.indexOf("media:") != -1){
		status = inputLine.remove(0, inputLine.indexOf("media:") + 7);
		status.truncate(inputLine.indexOf("\n") );
                if ( status.indexOf("(") != -1)
                {
                   status.remove(0, status.indexOf("(") + 1);
                }
                if ( status.indexOf(")") != -1)
                {
                   status.truncate(status.indexOf(")"));
                }

	}
	
	return status;
}

void ethernetconfig::slotCheckGlobalText(const QString &text )
{
   
  // Done, now set the apply button to off
  buttonApply->setEnabled(TRUE);
}







