/***************************************************************************
 *   Copyright (C) 2007 by Kris Moore   *
 *   kris@pcbsd.com   *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. *
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/


#include "ethernetconfig.h"
#include <kapplication.h>
#include <kuniqueapplication.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <QLocale>
#include <qtranslator.h>
#include <qdesktopwidget.h>
#include <qtextcodec.h>

static const char description[] = I18N_NOOP("Ethernet Setup GUI");

static const char version[] = "1.0";

int main(int argc, char **argv)
{
   KAboutData aboutData("ETHERGUI", 0, ki18n("ETHERGUI"),
                            "1.2", ki18n(description),
                            KAboutData::License_BSD,
                            ki18n("(c) 2008, Kris Moore(c)"));

    aboutData.addAuthor(ki18n("Kris Moore"), ki18n("Current maintainer"), "kris@pcbsd.org");

    KCmdLineArgs::init(argc, argv, &aboutData);

    KCmdLineOptions options;
    options.add("+<device>", ki18n("Start the Network configuration on specified ethernet device"));
    options.add("info <device>", ki18n("Show the info tab on the specified ethernet device."));



    // Tell which options are supported
    KCmdLineArgs::addCmdLineOptions( options );


    KApplication app;





    ethernetconfig *mainWin = 0;

    QTranslator translator( 0 );
    QLocale mylocale(0);
    translator.load( "ethernetconfig_" + mylocale.language(), KStandardDirs::locate("data", "pc-netmanager/locale/") );
    app.installTranslator( &translator );

     // no session.. just start up normally
     KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
     mainWin = new ethernetconfig();
    // app.setMainWidget( mainWin );

        if ( argc == 2)
        {
		QString Device = argv[1];
		mainWin->setDevice( Device );
		mainWin->setWindowTitle(Device + " " + "configuration");

       		// Center the dialog window
   		QDesktopWidget *d = KApplication::desktop();
   		mainWin->move(d->width() / 3, d->height() / 4);
	        mainWin->programInit();
	        mainWin->show();
	} else if ( argc == 3  ) {
	 QString val = argv[1];
	 if ( val != "info" ) {
	   exit(1);  
	 }
	 
	        QString Device = argv[2];
		mainWin->setDevice( Device );
		mainWin->setWindowTitle(Device + " " + "configuration");
		mainWin->showInfo();
       		// Center the dialog window
   		QDesktopWidget *d = KApplication::desktop();
   		mainWin->move(d->width() / 3, d->height() / 4);
	        mainWin->programInit();
		mainWin->show();
	} else {
		exit(1);
	}



        args->clear();

    // mainWin has WDestructiveClose flag by default, so it will delete itself.
    return app.exec();
}

