#include "wifiscanssid.h"
#include "ui_wifiscanssid.h"
#include <qtextstream.h>
#include <kstandarddirs.h>
#include <qtimer.h>

void wifiscanssid::init(QString device)
{
   DeviceName = device;
   pushSelect->setEnabled(FALSE);
    
   QTimer::singleShot(500,  this,  SLOT(slotRescan()));

   connect( pushRescan, SIGNAL( clicked() ), this, SLOT(slotRescan()) );
   connect( pushSelect, SIGNAL( clicked() ), this, SLOT(slotConnect()) );
   connect( pushCancel, SIGNAL( clicked() ), this, SLOT(slotCancel()) );
}
   
void wifiscanssid::scanWifi()
{
    QString tmp;
    QString strength;
    QString ssid;
    int newStrength;
    bool ok;
    int foundItem = 0;
    QString FileLoad;
    QString line;
    
    // Clear the list box
    listWifi->clear();
    textTop->setText(tr("Scanning for wireless networks...") );
            
    // Start the scan and get the output
    line = getLineFromCommandOutput("ifconfig -v " + DeviceName + " up list scan >/tmp/.wifilist 2>/tmp/.wifilist");

    // Now read the output file
    QFile file( "/tmp/.wifilist" );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &file );
        while ( !stream.atEnd() ) {
                 line = stream.readLine(); // line of text excluding '\n'
	     // exclude the header line
	     if ( line.indexOf("BSSID") == -1) {
		     
		 // Get the ssid of the network
		 tmp = line;
		 tmp = tmp.simplified();
		 tmp.remove(0, tmp.indexOf("SSID<") + 5);
		 tmp.truncate(tmp.indexOf(">"));
		 ssid = tmp;

		 // Get the signal strength of this device
		 tmp = line;
		 tmp = tmp.simplified();

                 // First remove the SSID which may have spaces in it
		 tmp.remove(0, tmp.indexOf(":"));
                  
                 // Now find the strength section
		 tmp.truncate(tmp.lastIndexOf("SSID<"));
		 tmp.truncate(tmp.lastIndexOf(":"));
		 tmp.remove(0, tmp.lastIndexOf(" "));
                 strength = tmp;
                 		
	         //QMessageBox::warning( this, "Testing", "SSID:" + ssid + " STR:" + strength );
		 // Add the device to the list box
		 strength.toInt(&ok);
		 if ( ok && !ssid.isEmpty() && ssid != " " ) {
		     newStrength = tmp.toInt(&ok);
		     if ( newStrength < 25 )
		     {	
			 FileLoad=KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi.png");
		     } else if ( newStrength < 50 ) {
			 FileLoad=KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi30.png");
		     }  else if ( newStrength < 75 ) {
			 FileLoad=KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi60.png");
		     } else {;
			 FileLoad=KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi85.png");
		     }
		 
		 
		       QImage *Icon = new QImage(FileLoad);
		       QPixmap PixmapIcon;
	                     PixmapIcon.fromImage(Icon->scaled(22,22));
		       listWifi->addItem(new QListWidgetItem(PixmapIcon, ssid + " (signal strength: " +strength + "%)") );
		       
		       foundItem = 1;
		 
	     } else {
		  if ( ! ssid.isEmpty() && ssid != " " ) {
		       // If we didn't get the signal strength, still display the SSID
		       FileLoad=KStandardDirs::locate("data", "pc-netmanager/pics/tray_wifi.png");
		       QImage *Icon = new QImage(FileLoad);
		       QPixmap PixmapIcon;
	                     PixmapIcon.fromImage(Icon->scaled(22,22));
		       listWifi->addItem(new QListWidgetItem(PixmapIcon, ssid + " (signal strength: unknown)") );
		       
		       foundItem = 1;
                  }
	     }
	    
	}
	
    }
	
	
	if ( foundItem == 1 )
	{
	    textTop->setText(tr("Select a wireless network to connect.") );
	    listWifi->setCurrentRow(0);
	    pushSelect->setEnabled(TRUE);
              } else {
                  textTop->setText(tr("No wireless networks found!") );
	    pushSelect->setEnabled(FALSE);
	}
    }
    
}



void wifiscanssid::slotCancel()
{
    close();
}


void wifiscanssid::slotConnect()
{
    QString selectedSSID;
    
    // Make sure a device was selected
    int sel = listWifi->currentRow();
    if ( sel == -1 )
    {
	QMessageBox::warning( this, tr("No selection"),
			    tr("Error: You must select a network to connect!\n") );
	return;
    }
    
    QListWidgetItem *selected = listWifi->currentItem(); 
    selectedSSID = selected->text();
    selectedSSID.truncate( selectedSSID.indexOf("(") - 1 );
    
    // Emit the new network SSID the user selected
    emit saved(selectedSSID);
    close();
}




void wifiscanssid::slotRescan()
{
    pushSelect->setEnabled(FALSE);
    scanWifi();
}

QString wifiscanssid::getLineFromCommandOutput( QString command )
{
        FILE *file = popen(command.toLatin1(),"r");

        char buffer[100];

        QString line = "";
        char firstChar;

        if ((firstChar = fgetc(file)) != -1){
                line += firstChar;
                line += fgets(buffer,100,file);
        }


        pclose(file);


        return line;
}


